package com.wily.jbley.supergnf;

import com.wily.introscope.agent.AgentNotAvailableException;
import com.wily.introscope.agent.AgentShim;
import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.trace.INameFormatter;
import com.wily.introscope.agent.trace.InvocationData;
import com.wily.util.StringUtils;

public class SuperGNF
    implements
        INameFormatter
{
    public static final String StartDelimiter = "{{";

    public static final String StopDelimiter  = "}}";

    // FIXME might want more flexible policy (e.g., allowing | to be computed in
    // the script)
    public static String makeSafeForMetrics(final String s)
    {
        String result = s.replace(':', '_').replace('|', '_');
        result = StringUtils.removeTrailingSpaces(result);
        return result.equals("") ? "<empty>" : result;
    }

    private final IAgent agent;

    public SuperGNF() throws AgentNotAvailableException
    {
        this(AgentShim.getAgent());
    }

    public SuperGNF(final IAgent agent)
    {
        this.agent = agent;
    }

    public String format(final String name, final InvocationData data) throws Exception
    {
        int pos = name.indexOf(StartDelimiter);
        if (pos == -1)
        {
            return name;
        }

        // FIXME might could use eclipse JDT instead to compile scripts down
        // Note that reusing the interpreter instance would cause namespace
        // pollution between scripts (including between calls to the same
        // script)
        final ScriptEnvironment scriptEnvironment = ScriptEnvironment.getInstance();
        final StringBuffer result = scriptEnvironment.stringBuffer;
        scriptEnvironment.interpreter.set("data", data);
        try
        {
            // FIXME might want some shorthand here like "param1" - note that
            // "this"
            // seems to cause problems
            int lastEndPos = 0;
            while (pos != -1)
            {
                result.append(name.substring(lastEndPos, pos));
                final int endPos = name.indexOf(StopDelimiter, pos);
                final String script = name.substring(pos + 2, endPos);
                String scriptResult = String.valueOf(scriptEnvironment.interpreter.eval(script));
                scriptResult = makeSafeForMetrics(scriptResult);

                result.append(scriptResult);
                // wrap and do it again
                lastEndPos = endPos + 2;
                pos = name.indexOf(StartDelimiter, endPos);
            }
            result.append(name.substring(lastEndPos, name.length()));

            return result.toString();
        } finally
        {
            scriptEnvironment.reset();
        }
    }

    public boolean INameFormatter_canUseRuntimeFullClassNameCaching()
    {
        return false;
    }

    public String INameFormatter_format(final InvocationData data,
                                        final String name)
    {
        return INameFormatter_format(name, data);
    }

    public String INameFormatter_format(final String name,
                                        final InvocationData data)
    {
        try
        {
            final long start = System.currentTimeMillis();

            final String answer = format(name, data);
            final long duration = System.currentTimeMillis() - start;
            if (duration > 500L)
            {
                this.agent.IAgent_getModuleFeedback()
                          .warn("[GenericNameFormatter] Metric name formatting for \""
                                + name
                                + "\" took "
                                + duration
                                + "ms");
            }

            return answer;
        } catch (final Exception e)
        {
            this.agent.IAgent_getModuleFeedback().error("SuperGNF: " + e, e);
        }
        return name;
    }

}
