package com.wily.jbley.supergnf;

import bsh.EvalError;
import bsh.Interpreter;

import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.errors.ErrorSnapshotFeatureFactory;
import com.wily.introscope.agent.stat.IDataAccumulator;
import com.wily.introscope.agent.stat.ILongIntervalCounterDataAccumulator;
import com.wily.introscope.agent.trace.ASingleMetricTracerFactory;
import com.wily.introscope.agent.trace.InvocationData;
import com.wily.introscope.agent.trace.ProbeIdentification;
import com.wily.util.properties.AttributeListing;

public class SuperGNFErrorReportingTracer
    extends ASingleMetricTracerFactory
{
    public static final String ErrorConditionKey = "ErrorCondition";

    public static final String ErrorMessageKey   = "ErrorMessage";

    public SuperGNFErrorReportingTracer(IAgent agent,
                                        AttributeListing parameters,
                                        ProbeIdentification probe,
                                        Object sampleTracedObject)
    {
        super(agent, parameters, probe, sampleTracedObject);
    }

    protected final IDataAccumulator createDataAccumulator(String formattedMetricName)
    {
        return getDataAccumulatorFactory().safeGetLongIntervalCounterDataAccumulator(formattedMetricName);
    }

    public String createErrorMessage(InvocationData data)
    {
        Object result = evalScriptFromKey(data, ErrorMessageKey);
        return String.valueOf(result);
    }

    private Object evalScriptFromKey(InvocationData data, String keyName)
    {
        String script = getParameter(keyName);
        if (script == null)
        {
            return null;
        }
        Interpreter beanshell = new Interpreter();
        try
        {
            beanshell.set("data", data);
            return beanshell.eval(script);
        } catch (EvalError e)
        {
            getModuleFeedback().error("Can't evaluate " + script + ": " + e, e);
        }
        return null;
    }

    public boolean hasError(InvocationData data)
    {
        Object result = evalScriptFromKey(data, ErrorConditionKey);
        return "true".equalsIgnoreCase(String.valueOf(result));
    }

    public void ITracer_finishTrace(int tracerIndex, InvocationData data)
    {
        if (hasError(data))
        {
            reportError(data);
        }
    }

    public void ITracer_startTrace(int tracerIndex, InvocationData data)
    {
        validateParameter(ErrorConditionKey);
        validateParameter(ErrorMessageKey);
    }

    public void reportError(InvocationData data)
    {
        String errorMessage = createErrorMessage(data);

        if (ErrorSnapshotFeatureFactory.flagErrorInManagedApplication(errorMessage))
        {
            ((ILongIntervalCounterDataAccumulator) getDataAccumulator(data)).ILongIntervalCounterDataAccumulator_addSingleIncident();
            getAgent().IAgent_getComponentTracer().noteBoundaryError();
        }
    }

    private void validateParameter(String key)
    {
        String script = getParameter(key);
        if ((script == null) || (script.length() == 0))
        {
            ProbeIdentification pi = getProbeIdentification();
            String id = pi.getProbeClassName()
                        + "."
                        + pi.getProbeMethodName()
                        + pi.getProbeMethodDescriptor();
            getModuleFeedback().error("SuperGNFErrorReportingTracer parameter "
                                      + key
                                      + " not set on "
                                      + id);
        }
    }

}
