package com.wily.jbley.supergnf.poll;

import java.lang.ref.WeakReference;

import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.trace.ITracerFactory;
import com.wily.introscope.agent.trace.InvocationData;
import com.wily.introscope.agent.trace.ProbeIdentification;
import com.wily.introscope.agent.trace.ProbeInformation;
import com.wily.jbley.supergnf.SuperGNF;

public class Poll
{

    private static final ITracerFactory[] NoTracers = new ITracerFactory[0];

    public final WeakReference            refInvocationObject;

    public final String                   gnfExprToPoll;

    public final String                   gnfExprForMetricName;

    public final ProbeIdentification      probeIdentification;

    public Poll(Object invocationObject,
                String gnfExprToPoll,
                String gnfExprForMetricName,
                ProbeIdentification pi)
    {
        this.refInvocationObject = new WeakReference(invocationObject);
        this.gnfExprToPoll = gnfExprToPoll;
        this.gnfExprForMetricName = gnfExprForMetricName;
        this.probeIdentification = pi;
    }

    public boolean equals(Object o)
    {
        if (o == this) // seems odd, but this is a really easy way to make
                       // remove() work - see the hashCode note
        {
            return true;
        }
        if (!(o instanceof Poll))
        {
            return false;
        }
        Poll that = (Poll) o;
        Object thisO = this.refInvocationObject.get();
        Object thatO = that.refInvocationObject.get();
        if ((thisO == null) || (thatO == null))
        {
            return false;
        }
        return this.gnfExprForMetricName.equals(that.gnfExprForMetricName)
               && this.gnfExprToPoll.equals(that.gnfExprToPoll)
               && (thisO == thatO); // identity, not equality
    }

    public int hashCode()
    {
        return hashCodeFor(this.gnfExprForMetricName)
               ^ hashCodeFor(this.gnfExprToPoll);
        // NOTE: the hashCode()s won't be the same after the referent gets
        // cleared
        // and since the HashSet caches the hash value we can't use the
        // refInvocationObject
        // in any way to determine hash or equality
        // Also can't use the ProbeIdentification point since, e.g., there could
        // be identical probes on
        // super and subclass constructors and we'd like to treat these the same
        // at runtime
        // for the same object instance
    }

    private int hashCodeFor(Object x)
    {
        return x == null ? 0 : x.hashCode();
    }

    public void invoke(IAgent agent)
    {
        Object invocationObject = this.refInvocationObject.get();
        ProbeInformation pi = new ProbeInformation(agent,
                                                   this.probeIdentification,
                                                   NoTracers);
        InvocationData fakeID = InvocationData.debug_createInvocationData(agent,
                                                                          pi,
                                                                          invocationObject);
        SuperGNF gnf = new SuperGNF(agent);
        String metricName = gnf.INameFormatter_format(this.gnfExprForMetricName,
                                                      fakeID);
        String stringValue = gnf.INameFormatter_format(this.gnfExprToPoll,
                                                       fakeID);
        int value = Integer.parseInt(stringValue);
        agent.IAgent_getDataAccumulatorFactory()
             .safeGetIntegerFluctuatingCounterDataAccumulator(metricName)
             .IIntegerCounterDataAccumulator_setValue(value);
    }

}
