package com.wily.jbley.supergnf.poll;

import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.trace.ASingleInstanceTracerFactory;
import com.wily.introscope.agent.trace.InvocationData;
import com.wily.introscope.agent.trace.ProbeIdentification;
import com.wily.introscope.agent.trace.ReentrancyLevel;
import com.wily.util.properties.AttributeListing;

public class PollingTracer
    extends ASingleInstanceTracerFactory
{
    public static final String PollExpressionTracerParameterKey = "PollExpression";

    public PollingTracer(IAgent agent,
                         AttributeListing parameters,
                         ProbeIdentification probe,
                         Object sampleTracedObject)
    {
        super(agent, parameters, probe, sampleTracedObject);
    }

    private void computeAndAddPoller(InvocationData data)
    {
        String gnfExprToPoll = getParameter(PollExpressionTracerParameterKey);
        String gnfExprForMetricName = getParameter(kNameKey);
        ProbeIdentification pi = getProbeIdentification();
        Poll p = new Poll(data.getInvocationObject(),
                          gnfExprToPoll,
                          gnfExprForMetricName,
                          pi);
        PollingService.addPoll(p);
    }

    public void ITracer_finishTrace(int tracerIndex, InvocationData data)
    {
        if (data.getInvocationObject() != null)
        {
            computeAndAddPoller(data);
        }
    }

    public void ITracer_startTrace(int tracerIndex, InvocationData data)
    {
        // just do some error checking; the instance reference is not available
        // until finishTrace
        String pollExpr = getParameter(PollExpressionTracerParameterKey);
        if ((pollExpr == null) || (pollExpr.length() == 0))
        {
            ProbeIdentification pi = getProbeIdentification();
            String id = pi.getProbeClassName()
                        + "."
                        + pi.getProbeMethodName()
                        + pi.getProbeMethodDescriptor();
            getModuleFeedback().error("PollingTracer parameter "
                                      + PollExpressionTracerParameterKey
                                      + " not set on "
                                      + id);
        }
    }

    public ReentrancyLevel ITracerFactory_getReentrancyLevel()
    {
        return ReentrancyLevel.kNone;
    }

    public boolean ITracerFactory_isShutoff()
    {
        return false;
    }

}
