package com.wily.jbley.supergnftests;

import junit.framework.TestCase;

import com.wily.introscope.agent.AgentShim;
import com.wily.introscope.agent.IAgent;
import com.wily.introscope.agent.trace.ITracerFactory;
import com.wily.introscope.agent.trace.InvocationData;
import com.wily.introscope.agent.trace.ProbeIdentification;
import com.wily.introscope.agent.trace.ProbeInformation;
import com.wily.jbley.supergnf.SuperGNF;

public class PerformanceTests
    extends TestCase
{

    private IAgent           agent;

    private ProbeInformation probeInfo;

    protected void setUp() throws Exception
    {
        super.setUp();
        this.agent = AgentShim.getAgent();
        this.probeInfo = new ProbeInformation(this.agent,
                                              new ProbeIdentification("probe.ClassName",
                                                                      "probeMethod",
                                                                      "()V",
                                                                      "runtime.ClassName"),
                                              new ITracerFactory[0]);
    }

    public void testPerformance() throws Exception
    {
        final Object thisObj = new PerformanceTests();
        final InvocationData data = InvocationData.debug_createInvocationData(this.agent,
                                                                              this.probeInfo,
                                                                              thisObj);
        final SuperGNF gnf = new SuperGNF();
        Thread.sleep(15000);
        final String[][] tests = new String[][] {
                {
                        "Foo:{{data.getInvocationObject().getClass().getName()}}",
                        "Foo:" + getClass().getName() },
                {
                        "Foo:{{data.getClass().getName()}}",
                        "Foo:" + InvocationData.class.getName() },
                { "Foo|{{4+1}}|{{2+2}}:Stuff", "Foo|5|4:Stuff" }, };
        final long startTime = System.currentTimeMillis();
        final int nLoops = 1000;
        for (int loop = 0; loop < nLoops; loop++)
        {
            for (int i = 0; i < tests.length; i++)
            {
                final String in = tests[i][0];
                final String expected = tests[i][1];
                final String result = gnf.format(in, data);
                assertEquals(expected, result);
            }
        }
        final long endTime = System.currentTimeMillis();
        System.out.println("Time: " + (endTime - startTime));
    }

}
